@@ -1,9 +1,6 @@
-// -----------------------------------------------------------------------------------------------------
-// Copyright (c) 2006-2021, Knut Reinert & Freie Universität Berlin
-// Copyright (c) 2016-2021, Knut Reinert & MPI für molekulare Genetik
-// This file may be used, modified and/or redistributed under the terms of the 3-clause BSD-License
-// shipped with this file and also available at: https://github.com/seqan/seqan3/blob/master/LICENSE.md
-// -----------------------------------------------------------------------------------------------------
+// SPDX-FileCopyrightText: 2006-2025 Knut Reinert & Freie Universität Berlin
+// SPDX-FileCopyrightText: 2016-2025 Knut Reinert & MPI für molekulare Genetik
+// SPDX-License-Identifier: BSD-3-Clause
 
 /*!\file
  * \author Svenja Mehringer <svenja.mehringer AT fu-berlin.de>
@@ -14,12 +11,12 @@
 
 #include <future>
 #include <iostream>
+#include <regex>
 #include <set>
 #include <sstream>
 #include <string>
 #include <variant>
 #include <vector>
-#include <regex>
 
 // #include <seqan3/argument_parser/detail/format_ctd.hpp>
 #include <seqan3/argument_parser/detail/format_help.hpp>
@@ -30,8 +27,8 @@
 #include <seqan3/argument_parser/detail/version_check.hpp>
 #include <seqan3/core/debug_stream/detail/to_string.hpp>
 #include <seqan3/core/detail/test_accessor.hpp>
-#include <seqan3/io/stream/concept.hpp>
-#include <seqan3/utility/char_operations/predicate.hpp>
+
+SEQAN3_DEPRECATED_HEADER("This header and its functionality is deprecated and will be removed in a future version of SeqAn. Please use the sharg-parser (url: https://github.com/seqan/sharg-parser) instead.");
 
 namespace seqan3
 {
@@ -137,10 +134,6 @@ namespace seqan3
  * which means that applications ship with less bugs.
  * For privacy implications, please see: https://github.com/seqan/seqan3/wiki/Update-Notifications.
  *
- * Developers that wish to disable this feature permanently can pass an extra constructor argument:
- *
- * \include doc/tutorial/argument_parser/disable_version_check.cpp
- *
  * Users of applications that have this feature activated can opt-out, by either:
  *
  *  * disabling it for a specific application simply by setting the option `--version-check false/0` or
@@ -156,30 +149,11 @@ public:
     /*!\name Constructors, destructor and assignment
      * \{
      */
-    argument_parser() = delete;                                     //!< Deleted.
-    argument_parser(argument_parser const &) = default;             //!< Defaulted.
-    argument_parser & operator=(argument_parser const &) = default; //!< Defaulted.
-    argument_parser(argument_parser &&) = default;                  //!< Defaulted.
-    argument_parser & operator=(argument_parser &&) = default;      //!< Defaulted.
-
-#ifdef SEQAN3_DEPRECATED_310
-    //!\deprecated Please use seqan3::update_notifications::on or seqan3::update_notifications::off for `version_updates`.
-    SEQAN3_DEPRECATED_310
-    argument_parser(std::string const app_name,
-                    int const argc,
-                    char const * const * const  argv,
-                    bool version_updates,
-                    std::vector<std::string> subcommands = {}) :
-        argument_parser
-        {
-            app_name,
-            argc,
-            argv,
-            version_updates ? update_notifications::on : update_notifications::off,
-            subcommands
-        }
-    {}
-#endif // SEQAN3_DEPRECATED_310
+    argument_parser() = delete;                                    //!< Deleted.
+    argument_parser(argument_parser const &) = delete;             //!< Deleted. Holds std::future.
+    argument_parser & operator=(argument_parser const &) = delete; //!< Deleted. Holds std::future.
+    argument_parser(argument_parser &&) = default;                 //!< Defaulted.
+    argument_parser & operator=(argument_parser &&) = default;     //!< Defaulted.
 
     /*!\brief Initializes an seqan3::argument_parser object from the command line arguments.
      *
@@ -191,10 +165,10 @@ public:
      *
      * \throws seqan3::design_error if the application name contains illegal characters.
      *
-     * The application name must only contain alpha-numeric characters, '_' or '-',
-     * i.e. the following regex must evaluate to true: `\"^[a-zA-Z0-9_-]+$\"`.
+     * The application name must only contain alpha-numeric characters, `_` or `-`,
+     * i.e. the following regex must evaluate to true: `"^[a-zA-Z0-9_-]+$"`.
      *
-     * See the [argument parser tutorial](https://docs.seqan.de/seqan/3-master-dev/tutorial_argument_parser.html)
+     * See the [argument parser tutorial](https://docs.seqan.de/seqan3/main_dev/tutorial_argument_parser.html)
      * for more information about the version check functionality.
      */
     argument_parser(std::string const app_name,
@@ -208,7 +182,7 @@ public:
         if (!std::regex_match(app_name, app_name_regex))
         {
             throw design_error{("The application name must only contain alpha-numeric characters or '_' and '-' "
-                               "(regex: \"^[a-zA-Z0-9_-]+$\").")};
+                                "(regex: \"^[a-zA-Z0-9_-]+$\").")};
         }
 
         for (auto & sub : this->subcommands)
@@ -220,7 +194,6 @@ public:
             }
         }
 
-
         info.app_name = std::move(app_name);
 
         init(argc, argv);
@@ -259,11 +232,9 @@ public:
      * \throws seqan3::design_error
      */
     template <typename option_type, validator validator_type = detail::default_validator<option_type>>
-    //!\cond
-        requires (argument_parser_compatible_option<option_type> ||
-                  argument_parser_compatible_option<std::ranges::range_value_t<option_type>>) &&
-                  std::invocable<validator_type, option_type>
-    //!\endcond
+        requires (argument_parser_compatible_option<option_type>
+                  || argument_parser_compatible_option<std::ranges::range_value_t<option_type>>)
+              && std::invocable<validator_type, option_type>
     void add_option(option_type & value,
                     char const short_id,
                     std::string const & long_id,
@@ -277,17 +248,24 @@ public:
         verify_identifiers(short_id, long_id);
         // copy variables into the lambda because the calls are pushed to a stack
         // and the references would go out of scope.
-        std::visit([=, &value] (auto & f) { f.add_option(value, short_id, long_id, desc, spec, option_validator); },
-                   format);
+        std::visit(
+            [=, &value](auto & f)
+            {
+                f.add_option(value, short_id, long_id, desc, spec, option_validator);
+            },
+            format);
     }
 
     /*!\brief Adds a flag to the seqan3::argument_parser.
      *
-     * \param[in, out] value     The variable in which to store the given command line argument.
+     * \param[in, out] value     The variable which shows whether the flag is turned off (default) or on.
      * \param[in]      short_id  The short identifier for the flag (e.g. 'i').
      * \param[in]      long_id   The long identifier for the flag (e.g. "integer").
      * \param[in]      desc      The description of the flag to be shown in the help page.
      * \param[in]      spec      Advanced flag specification, see seqan3::option_spec.
+     *
+     * \throws seqan3::design_error if `value` is true.
+     *
      */
     void add_flag(bool & value,
                   char const short_id,
@@ -295,10 +273,18 @@ public:
                   std::string const & desc,
                   option_spec const spec = option_spec::standard)
     {
+        if (value)
+            throw design_error("A flag's default value must be false.");
+
         verify_identifiers(short_id, long_id);
         // copy variables into the lambda because the calls are pushed to a stack
         // and the references would go out of scope.
-        std::visit([=, &value] (auto & f) { f.add_flag(value, short_id, long_id, desc, spec); }, format);
+        std::visit(
+            [=, &value](auto & f)
+            {
+                f.add_flag(value, short_id, long_id, desc, spec);
+            },
+            format);
     }
 
     /*!\brief Adds a positional option to the seqan3::argument_parser.
@@ -322,11 +308,9 @@ public:
      * The validator must be applicable to the given output variable (\p value).
      */
     template <typename option_type, validator validator_type = detail::default_validator<option_type>>
-    //!\cond
-        requires (argument_parser_compatible_option<option_type> ||
-                  argument_parser_compatible_option<std::ranges::range_value_t<option_type>>) &&
-                  std::invocable<validator_type, option_type>
-    //!\endcond
+        requires (argument_parser_compatible_option<option_type>
+                  || argument_parser_compatible_option<std::ranges::range_value_t<option_type>>)
+              && std::invocable<validator_type, option_type>
     void add_positional_option(option_type & value,
                                std::string const & desc,
                                validator_type option_validator = validator_type{}) // copy to bind rvalues
@@ -336,21 +320,26 @@ public:
 
         if (has_positional_list_option)
             throw design_error{"You added a positional option with a list value before so you cannot add "
-                                      "any other positional options."};
+                               "any other positional options."};
 
-        if constexpr (sequence_container<option_type> && !std::same_as<option_type, std::string>)
+        if constexpr (detail::is_container_option<option_type>)
             has_positional_list_option = true; // keep track of a list option because there must be only one!
 
         // copy variables into the lambda because the calls are pushed to a stack
         // and the references would go out of scope.
-        std::visit([=, &value] (auto & f) { f.add_positional_option(value, desc, option_validator); }, format);
+        std::visit(
+            [=, &value](auto & f)
+            {
+                f.add_positional_option(value, desc, option_validator);
+            },
+            format);
     }
     //!\}
 
     /*!\brief Initiates the actual command line parsing.
      *
      * \attention The function must be called at the very end of all parser
-     * related code and should be enclosed in a try catch block.
+     * related code and should be enclosed in a try catch block as the argument parser may throw.
      *
      * \throws seqan3::design_error if this function was already called before.
      *
@@ -378,8 +367,6 @@ public:
      *
      * Example:
      *
-     * \note Since the argument parser may throw, you should always wrap `parse()` into a try-catch block.
-     *
      * \include test/snippet/argument_parser/argument_parser_2.cpp
      *
      * The code above gives the following output when calling `--help`:
@@ -424,7 +411,8 @@ public:
         if (std::holds_alternative<detail::format_parse>(format) && !subcommands.empty() && sub_parser == nullptr)
         {
             throw too_few_arguments{detail::to_string("You either forgot or misspelled the subcommand! Please specify"
-                                                      " which sub-program you want to use: one of ", subcommands,
+                                                      " which sub-program you want to use: one of ",
+                                                      subcommands,
                                                       ". Use -h/--help for more information.")};
         }
 
@@ -436,7 +424,12 @@ public:
             app_version(std::move(app_version_prom));
         }
 
-        std::visit([this] (auto & f) { f.parse(info); }, format);
+        std::visit(
+            [this](auto & f)
+            {
+                f.parse(info);
+            },
+            format);
         parse_was_called = true;
     }
 
@@ -479,9 +472,7 @@ public:
      *   via `seqan3::argument_parser::add_option()` calls beforehand.
      */
     template <typename id_type>
-    //!\cond
         requires std::same_as<id_type, char> || std::constructible_from<std::string, id_type>
-    //!\endcond
     bool is_option_set(id_type const & id) const
     {
         if (!parse_was_called)
@@ -495,9 +486,9 @@ public:
         {
             if (short_or_long_id.size() == 1)
             {
-                throw design_error{"Long option identifiers must be longer than one character! If " + short_or_long_id +
-                                   "' was meant to be a short identifier, please pass it as a char ('') not a string"
-                                   " (\"\")!"};
+                throw design_error{"Long option identifiers must be longer than one character! If " + short_or_long_id
+                                   + "' was meant to be a short identifier, please pass it as a char ('') not a string"
+                                     " (\"\")!"};
             }
         }
 
@@ -521,7 +512,12 @@ public:
      */
     void add_section(std::string const & title, option_spec const spec = option_spec::standard)
     {
-        std::visit([&] (auto & f) { f.add_section(title, spec); }, format);
+        std::visit(
+            [&](auto & f)
+            {
+                f.add_section(title, spec);
+            },
+            format);
     }
 
     /*!\brief Adds an help page subsection to the seqan3::argument_parser.
@@ -532,7 +528,12 @@ public:
      */
     void add_subsection(std::string const & title, option_spec const spec = option_spec::standard)
     {
-        std::visit([&] (auto & f) { f.add_subsection(title, spec); }, format);
+        std::visit(
+            [&](auto & f)
+            {
+                f.add_subsection(title, spec);
+            },
+            format);
     }
 
     /*!\brief Adds an help page text line to the seqan3::argument_parser.
@@ -546,7 +547,12 @@ public:
      */
     void add_line(std::string const & text, bool is_paragraph = false, option_spec const spec = option_spec::standard)
     {
-        std::visit([&] (auto & f) { f.add_line(text, is_paragraph, spec); }, format);
+        std::visit(
+            [&](auto & f)
+            {
+                f.add_line(text, is_paragraph, spec);
+            },
+            format);
     }
 
     /*!\brief Adds an help page list item (key-value) to the seqan3::argument_parser.
@@ -567,11 +573,15 @@ public:
      *            Super important integer for age.
      *```
      */
-    void add_list_item(std::string const & key,
-                       std::string const & desc,
-                       option_spec const spec = option_spec::standard)
+    void
+    add_list_item(std::string const & key, std::string const & desc, option_spec const spec = option_spec::standard)
     {
-        std::visit([&] (auto & f) { f.add_list_item(key, desc, spec); }, format);
+        std::visit(
+            [&](auto & f)
+            {
+                f.add_list_item(key, desc, spec);
+            },
+            format);
     }
     //!\}
 
@@ -720,7 +730,6 @@ private:
 
         bool special_format_was_set{false};
 
-
         for (int i = 1, argv_len = argc; i < argv_len; ++i) // start at 1 to skip binary name
         {
             std::string arg{argv[i]};
@@ -763,7 +772,7 @@ private:
                 {
                     if (argv_len <= i + 1)
                         throw too_few_arguments{"Option --export-help must be followed by a value."};
-                    export_format = {argv[i+1]};
+                    export_format = argv[i + 1];
                 }
 
                 if (export_format == "html")
@@ -775,7 +784,7 @@ private:
                 //     format = detail::format_ctd{};
                 else
                     throw validation_error{"Validation failed for option --export-help: "
-                                            "Value must be one of [html, man]"};
+                                           "Value must be one of [html, man]"};
                 init_standard_options();
                 special_format_was_set = true;
             }
@@ -816,14 +825,14 @@ private:
     {
         add_subsection("Basic options:");
         add_list_item("\\fB-h\\fP, \\fB--help\\fP", "Prints the help page.");
-        add_list_item("\\fB-hh\\fP, \\fB--advanced-help\\fP",
-                                    "Prints the help page including advanced options.");
+        add_list_item("\\fB-hh\\fP, \\fB--advanced-help\\fP", "Prints the help page including advanced options.");
         add_list_item("\\fB--version\\fP", "Prints the version information.");
         add_list_item("\\fB--copyright\\fP", "Prints the copyright/license information.");
         add_list_item("\\fB--export-help\\fP (std::string)",
-                                    "Export the help page information. Value must be one of [html, man].");
+                      "Export the help page information. Value must be one of [html, man].");
         if (version_check_dev_decision == update_notifications::on)
-            add_list_item("\\fB--version-check\\fP (bool)", "Whether to check for the newest app version. Default: true.");
+            add_list_item("\\fB--version-check\\fP (bool)",
+                          "Whether to check for the newest app version. Default: true.");
     }
 
     /*!\brief Checks whether the long identifier has already been used before.
@@ -850,7 +859,7 @@ private:
      */
     void verify_identifiers(char const short_id, std::string const & long_id)
     {
-        auto constexpr allowed = is_alnum || is_char<'_'> || is_char<'@'>;
+        constexpr auto allowed = is_alnum || is_char<'_'> || is_char<'@'>;
 
         if (id_exists(short_id))
             throw design_error("Option Identifier '" + std::string(1, short_id) + "' was already used before.");
@@ -863,10 +872,13 @@ private:
         if (long_id.size() > 0 && is_char<'-'>(long_id[0]))
             throw design_error("First character of long ID cannot be '-'.");
 
-        std::for_each(long_id.begin(), long_id.end(), [&allowed] (char c)
+        std::for_each(long_id.begin(),
+                      long_id.end(),
+                      [&allowed](char c)
                       {
                           if (!(allowed(c) || is_char<'-'>(c)))
-                              throw design_error("Long identifiers may only contain alphanumeric characters, '_', '-', or '@'.");
+                              throw design_error(
+                                  "Long identifiers may only contain alphanumeric characters, '_', '-', or '@'.");
                       });
         if (detail::format_parse::is_empty_id(short_id) && detail::format_parse::is_empty_id(long_id))
             throw design_error("Option Identifiers cannot both be empty.");
